#include "rtcpcl_lrftopcl.h"

#include <ctime>
#include <iostream>
#include <cmath>
#include <pcl/io/pcd_io.h>
#include <rtcpcl/pc_mgmt.h>
#include <stdlib.h>

using namespace RTCPCL;


RTCPCL_LRFtoPCL::RTCPCL_LRFtoPCL(RTC::Manager* manager)
    : RTC::DataFlowComponentBase(manager),
    corba_out_port_("PCL_Out0", corba_out_),
    PCL1_Out_port_("PCL_Out1", PCL1_Out_),
     m_LRF0_RangeIn_port_("LRF0_In",  m_LRF0_RangeIn_),
     LRF0_Pose_port_("LRF0_Pose",  LRF0_Pose_),
     m_LRF1_RangeIn_port_("LRF1_In",  m_LRF1_RangeIn_),
     LRF1_Pose_port_("LRF1_Pose",  LRF1_Pose_),
    seq_(0)
{
}


RTCPCL_LRFtoPCL::~RTCPCL_LRFtoPCL()
{
}


RTC::ReturnCode_t RTCPCL_LRFtoPCL::onInitialize()
{
    addOutPort(corba_out_port_.getName(), corba_out_port_);
    addOutPort("PCL_Out1",  PCL1_Out_port_);
    addInPort(m_LRF0_RangeIn_port_.getName(), m_LRF0_RangeIn_port_);
    addInPort("LRF0_Pose",  LRF0_Pose_port_);
    addInPort(m_LRF1_RangeIn_port_.getName(), m_LRF1_RangeIn_port_);
    addInPort("LRF1_Pose",  LRF1_Pose_port_);
    return RTC::RTC_OK;
}


RTC::ReturnCode_t RTCPCL_LRFtoPCL::onActivated(RTC::UniqueId ec_id)
{
    seq_ = 0;

    cloud_.reset(new pcl::PointCloud<pcl::PointXYZRGB>);
    double steps(length_ / step_);
    uint8_t r(255), g(0), b(0), colour_step(255 / steps);
    for (float z(-length_ / 2.0); z <= length_ / 2.0; z += step_)
    {
        for (float angle(0.0); angle <= 2 * M_PI; angle += angle_step_)
        {
            pcl::PointXYZRGB point;
            point.x = x_radius_ * cosf(angle);
            point.y = y_radius_ * sinf(angle);
            point.z = z;
            uint32_t rgb = (static_cast<uint32_t>(r) << 16 |
                    static_cast<uint32_t>(g) << 8 | static_cast<uint32_t>(b));
            point.rgb = *reinterpret_cast<float*>(&rgb);
            cloud_->points.push_back(point);
        }
        if (z < 0.0)
        {
            r -= colour_step;
            g += colour_step;
            b = 0;
        }
        else
        {
            r = 0;
            g -= colour_step;
            b += colour_step;
        }
    }

    cloud_->width = cloud_->points.size();
    cloud_->height = 1;
    return RTC::RTC_OK;
}


RTC::ReturnCode_t RTCPCL_LRFtoPCL::onDeactivated(RTC::UniqueId ec_id)
{
    cloud_.reset();
#if defined(DDS_SUPPORT)
    PointCloudTypes_PointCloudTypeSupport::delete_data(dds_out_);
#endif // defined(DDS_SUPPORT)
    return RTC::RTC_OK;
}


RTC::ReturnCode_t RTCPCL_LRFtoPCL::onExecute(RTC::UniqueId ec_id)
{
	int STEP=0;
	double STEP_RAD=0.0;
	if (corba_){
		if(m_LRF0_RangeIn_port_.isNew()){
			m_LRF0_RangeIn_port_.read();
			STEP=(int ) ((m_LRF0_RangeIn_.config.maxAngle*2)/m_LRF0_RangeIn_.config.angularRes);
			cloud_.reset(new pcl::PointCloud<pcl::PointXYZRGB>);
			double steps(length_ / step_);
			uint8_t r(255), g(0), b(0), colour_step(255 / steps);
			for(int i=0;i<STEP;i++){
				STEP_RAD=(i*m_LRF0_RangeIn_.config.angularRes-135*3.1415926/180);
				pcl::PointXYZRGB point;
				point.x = m_LRF0_RangeIn_.ranges[i]*sin(STEP_RAD)*100;
				point.y = m_LRF0_RangeIn_.ranges[i]*cos(STEP_RAD)*100;
				point.z = 0;
				uint32_t rgb = (static_cast<uint32_t>(r) << 16 |
				static_cast<uint32_t>(g) << 8 | static_cast<uint32_t>(b));
				point.rgb = *reinterpret_cast<float*>(&rgb);
				cloud_->points.push_back(point);
				cloud_->width = cloud_->points.size();
				cloud_->height = 1;
			}
			//		std::cout<<m_LRF0_RangeIn_.config.maxAngle<<std::endl;
			//		std::cout<<m_LRF0_RangeIn_.config.angularRes<<std::endl;
			//		std::cout<<STEP<<std::endl;

		pcl_to_pointcloud<pcl::PointXYZRGB>(cloud_, corba_out_);
		corba_out_port_.write();
		}
		if(m_LRF1_RangeIn_port_.isNew()){
			m_LRF1_RangeIn_port_.read();
			STEP=(int ) ((m_LRF1_RangeIn_.config.maxAngle*2)/m_LRF1_RangeIn_.config.angularRes);
			cloud_.reset(new pcl::PointCloud<pcl::PointXYZRGB>);
			double steps(length_ / step_);
			uint8_t r(255), g(0), b(0), colour_step(255 / steps);
			for(int i=0;i<STEP;i++){
				STEP_RAD=(i*m_LRF1_RangeIn_.config.angularRes-135*3.1415926/180);
				pcl::PointXYZRGB point;
				point.x = m_LRF1_RangeIn_.ranges[i]*sin(STEP_RAD)*100;
				point.y = m_LRF1_RangeIn_.ranges[i]*cos(STEP_RAD)*100;
				point.z = 0;
				uint32_t rgb = (static_cast<uint32_t>(r) << 16 |
				static_cast<uint32_t>(g) << 8 | static_cast<uint32_t>(b));
				point.rgb = *reinterpret_cast<float*>(&rgb);
				cloud_->points.push_back(point);
				cloud_->width = cloud_->points.size();
				cloud_->height = 1;
			}
			//		std::cout<<m_LRF1_RangeIn_.config.maxAngle<<std::endl;
			//		std::cout<<m_LRF1_RangeIn_.config.angularRes<<std::endl;
			//		std::cout<<STEP<<std::endl;
		pcl_to_pointcloud<pcl::PointXYZRGB>(cloud_, PCL1_Out_);
		PCL1_Out_port_.write();
		}
	}
	#if defined(DDS_SUPPORT)
	if (dds_)
	{
	pcl_to_pointcloud<pcl::PointXYZRGB>(cloud_, *dds_out_, true);
	dds_out_port_.write(*dds_out_);
	return_loan(*dds_out_);
	}
	#endif // defined(DDS_SUPPORT)






	return RTC::RTC_OK;
}


static const char* spec[] =
{
    "implementation_id", "RTCPCL_LRFtoPCL",
    "type_name",         "RTCPCL_LRFtoPCL",
    "description",       "RTCPCL_LRFtoPCL",
    "version",           "1.0",
    "vendor",            "AIZUK",
    "category",          "PCL",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    ""
};

extern "C"
{
    void rtc_init(RTC::Manager* manager)
    {
        coil::Properties profile(spec);
        manager->registerFactory(profile, RTC::Create<RTCPCL_LRFtoPCL>,
                RTC::Delete<RTCPCL_LRFtoPCL>);
    }
};

